<?xml version="1.0"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="html"/>
    <xsl:template match="/">
        <html>
            <head>
                <style>
                html, body, div, table, tr, td, h1, h2 {
                    padding: 0;
                    margin: 0;
                }
                html {
                    font-family: monospace;
                }
                main {
                    width: 80vw;
                    margin: 1em auto;
                }
                h1 {
                    text-align: center;
                }
                .movie {
                    margin-top: 1em;
                }
                h2 {
                    margin-bottom: 0.2em;
                    padding-left: 0.5rem;
                }
                table {
                    border-collapse: collapse;
                    table-layout: fixed;
                    width: 100%;
                }
                tr {
                    border-top: 1px solid black;
                }
                tr:first-child {
                    border: none;
                }
                tr:nth-child(2n) {
                    background-color: #eee;
                }
                td {
                    border-left: 1px solid black;
                    padding: 0.1em 0.5em;
                }
                td:first-child {
                    border: none;
                }
                .error {
                    width: 7ch;
                    color: #a11;
                }
                .success {
                    width: 7ch;
                    color: #1a1;
                }
                .time {
                    max-width: 16ch;
                }
                .msg {
                    width: calc(80vw - 16ch - 7ch - 3em - 2px);
                }
                </style>
                <title>Schedule feedback</title>
            </head>
            <body>
                <main>
                    <h1>Schedule feedback</h1>
                    <xsl:for-each select="schedules/schedule">
                        <div class="movie">
                            <h2>
                                Movie <xsl:value-of select="movie_id" />
                                at <xsl:value-of select="start" />
                                in hall <xsl:value-of select="hall_id" />
                            </h2>
                            <table>
                                <xsl:for-each select="success|error">
                                    <tr>
                                    <xsl:choose>
                                        <xsl:when test="local-name()='success'">
                                            <td class="success">success</td>
                                            <td class="time">
                                                <xsl:value-of select="." />
                                            </td>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <td class="error">error</td>
                                            <td class="time">
                                                <xsl:value-of select="time" />
                                            </td>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                        <xsl:if test="msg">
                                            <td class="msg">
                                                <xsl:value-of select="msg" />
                                            </td>
                                        </xsl:if>
                                    </tr>
                                </xsl:for-each>
                            </table>
                        </div>
                    </xsl:for-each>
                </main>
            </body>
        </html>
    </xsl:template>
</xsl:stylesheet>
